// tslint:disable-next-line:max-line-length
import { AfterViewInit, Component, ContentChildren, Directive, ElementRef, Input, OnInit, QueryList, TemplateRef, ViewChild, ViewChildren, HostListener } from '@angular/core';
import { CarouselDirective } from './../../directives/carousel.directive';
import { animate, AnimationBuilder, AnimationFactory, AnimationPlayer, style } from '@angular/animations';

@Directive({
  selector: '.carousel-item'
})
export class CarouselItemElement {
}
@HostListener('window:resize', ['$event'])
@Component({
  selector: 'carousel',
  exportAs: 'carousel',
  templateUrl: './carousel.html',
  styleUrls: ['./carousel.scss']
})
export class CarouselComponent implements AfterViewInit {
  @ContentChildren(CarouselDirective) items: QueryList<CarouselDirective>;
  @ViewChildren(CarouselItemElement, { read: ElementRef }) private itemsElements: QueryList<ElementRef>;
  @ViewChild('carousel', {static: false}) private carousel: ElementRef;
  @Input() timing = '750ms ease-in';
  @Input() showControls = true;
  private player: AnimationPlayer;
  private itemWidth: number;
  private currentSlide = 0;
  carouselWrapperStyle = {};

  next() {
    if (this.currentSlide + 1 === this.items.length) {
        this.currentSlide = -1;
    }
    this.currentSlide = (this.currentSlide + 1) % this.items.length;
    const offset = this.currentSlide * this.itemWidth;
    const myAnimation: AnimationFactory = this.buildAnimation(offset);
    this.player = myAnimation.create(this.carousel.nativeElement);
    this.player.play();
  }

  private buildAnimation(offset) {
    return this.builder.build([
      animate(this.timing, style({ transform: `translateX(-${offset}px)` }))
    ]);
  }

  constructor(private builder: AnimationBuilder) {
    this.itemWidth = window.innerWidth;
  }

  ngAfterViewInit() {
    // For some reason only here I need to add setTimeout, in my local env it's working without this.
    setTimeout(() => {
      // this.itemWidth = this.itemsElements.first.nativeElement.getBoundingClientRect().width;
      this.itemWidth = window.innerHeight;
      console.log(this.itemWidth);
      this.carouselWrapperStyle = {
        width: `${this.itemWidth}px`
      };
    });

    setInterval(() => {
      this.itemWidth = window.innerWidth;
      this.next();
    }, 3000);

  }

}
